package ESpice.diagram.part;

import org.eclipse.core.runtime.Platform;
import org.eclipse.emf.ecore.EAnnotation;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.gmf.runtime.notation.Diagram;
import org.eclipse.gmf.runtime.notation.View;

/**
 * This registry is used to determine which type of visual object should be
 * created for the corresponding Diagram, Node, ChildNode or Link represented
 * by a domain model object.
 * 
 * @generated
 */
public class ESpiceVisualIDRegistry {

	/**
	 * @generated
	 */
	private static final String DEBUG_KEY = "ESpice.diagram/debug/visualID"; //$NON-NLS-1$

	/**
	 * @generated
	 */
	public static int getVisualID(View view) {
		if (view instanceof Diagram) {
			if (ESpice.diagram.edit.parts.OrganizationEditPart.MODEL_ID
					.equals(view.getType())) {
				return ESpice.diagram.edit.parts.OrganizationEditPart.VISUAL_ID;
			} else {
				return -1;
			}
		}
		return ESpice.diagram.part.ESpiceVisualIDRegistry.getVisualID(view
				.getType());
	}

	/**
	 * @generated
	 */
	public static String getModelID(View view) {
		View diagram = view.getDiagram();
		while (view != diagram) {
			EAnnotation annotation = view.getEAnnotation("Shortcut"); //$NON-NLS-1$
			if (annotation != null) {
				return (String) annotation.getDetails().get("modelID"); //$NON-NLS-1$
			}
			view = (View) view.eContainer();
		}
		return diagram != null ? diagram.getType() : null;
	}

	/**
	 * @generated
	 */
	public static int getVisualID(String type) {
		try {
			return Integer.parseInt(type);
		} catch (NumberFormatException e) {
			if (Boolean.TRUE.toString().equalsIgnoreCase(
					Platform.getDebugOption(DEBUG_KEY))) {
				ESpice.diagram.part.ESpiceDiagramEditorPlugin.getInstance()
						.logError(
								"Unable to parse view type as a visualID number: "
										+ type);
			}
		}
		return -1;
	}

	/**
	 * @generated
	 */
	public static String getType(int visualID) {
		return String.valueOf(visualID);
	}

	/**
	 * @generated
	 */
	public static int getDiagramVisualID(EObject domainElement) {
		if (domainElement == null) {
			return -1;
		}
		if (ESpice.ESpicePackage.eINSTANCE.getOrganization().isSuperTypeOf(
				domainElement.eClass())
				&& isDiagram((ESpice.Organization) domainElement)) {
			return ESpice.diagram.edit.parts.OrganizationEditPart.VISUAL_ID;
		}
		return -1;
	}

	/**
	 * @generated
	 */
	public static int getNodeVisualID(View containerView, EObject domainElement) {
		if (domainElement == null) {
			return -1;
		}
		String containerModelID = ESpice.diagram.part.ESpiceVisualIDRegistry
				.getModelID(containerView);
		if (!ESpice.diagram.edit.parts.OrganizationEditPart.MODEL_ID
				.equals(containerModelID)) {
			return -1;
		}
		int containerVisualID;
		if (ESpice.diagram.edit.parts.OrganizationEditPart.MODEL_ID
				.equals(containerModelID)) {
			containerVisualID = ESpice.diagram.part.ESpiceVisualIDRegistry
					.getVisualID(containerView);
		} else {
			if (containerView instanceof Diagram) {
				containerVisualID = ESpice.diagram.edit.parts.OrganizationEditPart.VISUAL_ID;
			} else {
				return -1;
			}
		}
		switch (containerVisualID) {
		case ESpice.diagram.edit.parts.OrganizationEditPart.VISUAL_ID:
			if (ESpice.ESpicePackage.eINSTANCE.getWiderstand().isSuperTypeOf(
					domainElement.eClass())) {
				return ESpice.diagram.edit.parts.WiderstandEditPart.VISUAL_ID;
			}
			if (ESpice.ESpicePackage.eINSTANCE.getKnoten().isSuperTypeOf(
					domainElement.eClass())) {
				return ESpice.diagram.edit.parts.KnotenEditPart.VISUAL_ID;
			}
			if (ESpice.ESpicePackage.eINSTANCE.getStromquelle().isSuperTypeOf(
					domainElement.eClass())) {
				return ESpice.diagram.edit.parts.StromquelleEditPart.VISUAL_ID;
			}
			break;
		}
		return -1;
	}

	/**
	 * @generated
	 */
	public static boolean canCreateNode(View containerView, int nodeVisualID) {
		String containerModelID = ESpice.diagram.part.ESpiceVisualIDRegistry
				.getModelID(containerView);
		if (!ESpice.diagram.edit.parts.OrganizationEditPart.MODEL_ID
				.equals(containerModelID)) {
			return false;
		}
		int containerVisualID;
		if (ESpice.diagram.edit.parts.OrganizationEditPart.MODEL_ID
				.equals(containerModelID)) {
			containerVisualID = ESpice.diagram.part.ESpiceVisualIDRegistry
					.getVisualID(containerView);
		} else {
			if (containerView instanceof Diagram) {
				containerVisualID = ESpice.diagram.edit.parts.OrganizationEditPart.VISUAL_ID;
			} else {
				return false;
			}
		}
		switch (containerVisualID) {
		case ESpice.diagram.edit.parts.WiderstandEditPart.VISUAL_ID:
			if (ESpice.diagram.edit.parts.WiderstandIdEditPart.VISUAL_ID == nodeVisualID) {
				return true;
			}
			break;
		case ESpice.diagram.edit.parts.StromquelleEditPart.VISUAL_ID:
			if (ESpice.diagram.edit.parts.StromquelleIdEditPart.VISUAL_ID == nodeVisualID) {
				return true;
			}
			break;
		case ESpice.diagram.edit.parts.OrganizationEditPart.VISUAL_ID:
			if (ESpice.diagram.edit.parts.WiderstandEditPart.VISUAL_ID == nodeVisualID) {
				return true;
			}
			if (ESpice.diagram.edit.parts.KnotenEditPart.VISUAL_ID == nodeVisualID) {
				return true;
			}
			if (ESpice.diagram.edit.parts.StromquelleEditPart.VISUAL_ID == nodeVisualID) {
				return true;
			}
			break;
		}
		return false;
	}

	/**
	 * @generated
	 */
	public static int getLinkWithClassVisualID(EObject domainElement) {
		if (domainElement == null) {
			return -1;
		}
		return -1;
	}

	/**
	 * User can change implementation of this method to handle some specific
	 * situations not covered by default logic.
	 * 
	 * @generated
	 */
	private static boolean isDiagram(ESpice.Organization element) {
		return true;
	}

}
